@@ -5,11 +5,13 @@ import pickle
 from datetime import datetime, timedelta, timezone
 from typing import Union
 
+import numpy as np
 import pandas as pd
 from powerbot_client import ApiClient, ContractApi
 
 from powerbot_backtesting.data_acquisition import get_public_trades_by_days
 from powerbot_backtesting.exceptions import NotInCacheError
+from powerbot_backtesting.models import HistoryApiClient
 from powerbot_backtesting.utils import _find_cache, _cache_data, _orderbook_data_transformation, _delta_filter
 from powerbot_backtesting.utils.constants import *
 
@@ -27,24 +29,38 @@ def get_orders(contract_hist_data: dict[str, pd.DataFrame]) -> dict[str, list[di
 	order_list = {}
 
 	for key, value in contract_hist_data.items():
+		value.replace(np.nan, 0, inplace=True)
 		bids_asks = []
-		orders_all = value["orders"].to_list()
-		for nr, orders in enumerate(orders_all):
-			for k, v in orders.items():
-				if v:
-					if k == "bid":
+		if "orders" in value:
+			orders_all = value["orders"].to_list()
+			for nr, orders in enumerate(orders_all):
+				for k, v in orders.items():
+					if v and k in ["bid", "ask"]:
 						for x in v:
-							x["type"] = "bid"
-							x["best_bid_price"] = value.loc[nr, "best_bid_price"]
-							x["best_bid_quantity"] = value.loc[nr, "best_bid_quantity"]
-							x["vwap"] = value.loc[nr, "vwap"]
+							x["type"] = k
+							x["best_bid"] = round(value.loc[nr, "best_bid_price"], 2)
+							x["best_bid_qty"] = round(value.loc[nr, "best_bid_quantity"], 2)
+							x["best_ask"] = round(value.loc[nr, "best_ask_price"], 2)
+							x["best_ask_qty"] = round(value.loc[nr, "best_ask_quantity"], 2)
+							x["vwap"] = round(value.loc[nr, "vwap"], 2)
+							x["as_of"] = value.loc[nr]["as_of"].tz_convert(timezone.utc) if value.loc[nr]["as_of"].tzinfo else \
+							value.loc[nr]["as_of"].tz_localize(timezone.utc)
 							bids_asks.append(x)
-					if k == "ask":
+		else:
+			value = [v.to_dict() for r, v in value.iterrows()]
+			for nr, coll in enumerate(value):
+				for k, v in coll.items():
+					if v and k in ["bids", "asks"]:
 						for x in v:
-							x["type"] = "ask"
-							x["best_ask_price"] = value.loc[nr, "best_ask_price"]
-							x["best_ask_quantity"] = value.loc[nr, "best_ask_quantity"]
-							x["vwap"] = value.loc[nr, "vwap"]
+							x["type"] = k[:-1]
+							x["best_bid"] = round(value[nr]["best_bid"], 2)
+							x["best_bid_qty"] = round(value[nr]["best_bid_qty"], 2)
+							x["best_ask"] = round(value[nr]["best_ask"], 2)
+							x["best_ask_qty"] = round(value[nr]["best_ask_qty"], 2)
+							x["vwap"] = round(value[nr]["vwap"], 2)
+							x["as_of"] = value[nr]["as_of"].tz_convert(timezone.utc) if value[nr]["as_of"].tzinfo else \
+								value[nr]["as_of"].tz_localize(timezone.utc)
+							bids_asks.append(x)
 							bids_asks.append(x)
 
 		order_list[key] = bids_asks
@@ -53,11 +69,10 @@ def get_orders(contract_hist_data: dict[str, pd.DataFrame]) -> dict[str, list[di
 
 
 def get_ohlc_data(trade_data: dict[str, pd.DataFrame],
-                  delivery_area: str,
                   timesteps: int,
                   time_unit: str,
-                  api_client: ApiClient = None,
-                  sql_export: bool = False,
+                  delivery_area: str = None,
+                  api_client: Union[ApiClient, HistoryApiClient] = None,
                   use_cached_data: bool = True,
                   caching: bool = True,
                   gzip_files: bool = True,
@@ -67,11 +82,10 @@ def get_ohlc_data(trade_data: dict[str, pd.DataFrame],
 
 	Args:
 		trade_data (dict{key: DataFrame}): Dictionary of Dataframes containing Contract Trade Data
-		delivery_area (str): Area Code for Delivery Area
 		timesteps (int): Timesteps to group Trades by
 		time_unit (str): Time units for timesteps (either hours, minutes or seconds)
+		delivery_area (str): Area Code for Delivery Area (not needed when loading from historic cache)
 		api_client: PowerBot ApiClient
-		sql_export (bool): True if data was exported from SQL
 		use_cached_data (bool): If True, function tries to load data from cache wherever possible
 		caching (bool): True if data should be cached
 		gzip_files (bool): True if cached files should be gzipped
@@ -80,15 +94,14 @@ def get_ohlc_data(trade_data: dict[str, pd.DataFrame],
 	Returns:
 		dict{key: DataFrame}: Dictionary of DataFrames
 	"""
-	if not api_client and not sql_export:
-		raise Exception("API data exports need a valid API client")
-
 	# Setup Parameters
 	all_ohlc_data = {}
-	host = api_client.configuration.host if api_client else None
-	environment = host.split("/")[2].split(".")[0] if host else "sql"
-	exchange = host.split("/")[4] if host else list(trade_data.values())[0].exchange[0]
+	host = api_client.configuration.host if isinstance(api_client, ApiClient) else None
+	environment = host.split("/")[2].split(".")[0] if host else "prod"
+	exchange = host.split("/")[4] if host else api_client.exchange if isinstance(api_client, HistoryApiClient) else \
+	list(trade_data.values())[0].exchange[0]
 	file_ending = ".gz" if gzip_files else ""
+	delivery_area = api_client.delivery_area if isinstance(api_client, HistoryApiClient) else delivery_area
 
 	# Check if __cache__ already exists
 	cache_path = _find_cache()
@@ -126,7 +139,7 @@ def get_ohlc_data(trade_data: dict[str, pd.DataFrame],
 	# Cache Data as JSON
 	if caching:
 		_cache_data("ohlc", all_ohlc_data, delivery_area, exchange=exchange, api_client=api_client,
-					gzip_files=gzip_files, timesteps=timesteps, time_unit=time_unit[0], as_csv=False)
+		            gzip_files=gzip_files, timesteps=timesteps, time_unit=time_unit[0], as_csv=False)
 
 	# Cache Data as JSON
 	if caching:
@@ -184,45 +197,46 @@ def get_ohlc_data(trade_data: dict[str, pd.DataFrame],
 
 
 def get_orderbooks(contract_hist_data: dict[str, pd.DataFrame],
-				   delivery_area: str,
-				   timesteps: int = 15,
-				   time_unit: str = "minutes",
-				   timestamp: list[datetime] = None,
-				   from_timestamp: bool = False,
-				   api_client: ApiClient = None,
-				   use_cached_data: bool = True,
-				   caching: bool = True,
-				   as_json: bool = False) -> dict[str, pd.DataFrame]:
+                   delivery_area: str = None,
+                   timesteps: int = 15,
+                   time_unit: str = "minutes",
+                   timestamp: list[datetime] = None,
+                   from_timestamp: bool = False,
+                   api_client: Union[ApiClient, HistoryApiClient] = None,
+                   use_cached_data: bool = True,
+                   caching: bool = True,
+                   as_json: bool = False) -> dict[str, pd.DataFrame]:
 	"""
-	Converts contract history data into orderbooks in the specified timesteps. If no API client is passed, the function
+	Converts contract history data into order books in the specified timesteps. If no API client is passed, the function
 	will automatically assume that the data is production data.
 
 	Please be aware that optimally only data from one exchange at a time should be used (e.g. only EPEX).
 
-	To generate specific orderbooks for a position closing algorithm, the timestamp and from_timestamp parameters can
+	To generate specific order books for a position closing algorithm, the timestamp and from_timestamp parameters can
 	be used.
 
 	Args:
 		contract_hist_data (dict{key: DataFrame}): Dictionary of Dataframes containing Contract History Data
-		delivery_area (str): Area Code for Delivery Area
-		timesteps (int): Timesteps to group Orderbooks by
+		delivery_area (str): Area Code for Delivery Area (not needed when loading from historic cache)
+		timesteps (int): Timesteps to group order books by
 		time_unit (str): Time units for timesteps (either hours, minutes or seconds)
-		timestamp (list[datetime]): List of timestamps to generate orderbooks at/ from
-		from_timestamp (bool): True if timestamp serves as starting point for orderbook generation
+		timestamp (list[datetime]): List of timestamps to generate order books at/ from
+		from_timestamp (bool): True if timestamp serves as starting point for order book generation
 		api_client: PowerBot ApiClient
 		use_cached_data (bool): If True, function tries to load data from cache wherever possible
-		caching (bool): True if single orderbooks should be cached as JSON
-		as_json (bool): True if complete orderbook should be cached as JSON
+		caching (bool): True if single order books should be cached as JSON
+		as_json (bool): True if complete order book should be cached as JSON
 
 	Returns:
 		dict{key: DataFrame}: Dictionary of DataFrames
 	"""
 	# Setup
 	all_order_books = {}
-
-	host = api_client.configuration.host if api_client else None
+	host = api_client.configuration.host if isinstance(api_client, ApiClient) else None
 	environment = host.split("/")[2].split(".")[0] if host else "prod"
-	exchange = host.split("/")[4] if host else list(contract_hist_data.values())[0].exchange[0]
+	exchange = host.split("/")[4] if host else api_client.exchange if isinstance(api_client, HistoryApiClient) else \
+	list(contract_hist_data.values())[0].exchange[0]
+	delivery_area = api_client.delivery_area if isinstance(api_client, HistoryApiClient) else delivery_area
 
 	# Parameters
 	contract_times = sorted([i for i in [*contract_hist_data]])
@@ -283,14 +297,14 @@ def get_orderbooks(contract_hist_data: dict[str, pd.DataFrame],
 
 
 async def __process_orderbook(key: str,
-							  value: pd.DataFrame,
-							  directory: str,
-							  timesteps: int,
-							  time_unit: str,
-							  timestamp: Union[datetime, None],
-							  from_timestamp: bool,
-							  orderbook_dict: dict[str, pd.DataFrame],
-							  use_cached_data: bool):
+                              value: pd.DataFrame,
+                              directory: str,
+                              timesteps: int,
+                              time_unit: str,
+                              timestamp: Union[datetime, None],
+                              from_timestamp: bool,
+                              orderbook_dict: dict[str, pd.DataFrame],
+                              use_cached_data: bool):
 	"""
 	Asyncio function to process single order book. Return value is appended to collection of order books.
 
@@ -315,8 +329,11 @@ async def __process_orderbook(key: str,
 
 	except (NotInCacheError, FileNotFoundError):
 		# Filter out emtpy revisions
-		order_filter = value.orders.map(lambda x: x if x["bid"] or x["ask"] else None)
-		value = value.loc[~value.index.isin(order_filter[order_filter.isna()].index)]
+		if "orders" in value:
+			order_filter = value.orders.map(lambda x: x if x["bid"] or x["ask"] else None)
+			value = value.loc[~value.index.isin(order_filter[order_filter.isna()].index)]
+		else:
+			value = value.loc[~(value.bids.isna()) & ~(value.asks.isna())]
 
 		# Setting Either Starting Point or Specific Timestamp
 		start_time = value.as_of.min().replace(minute=0, second=0, microsecond=0, tzinfo=timezone.utc)
@@ -530,8 +547,7 @@ def calc_single_vwap(objects: dict[str, pd.DataFrame],
 	vwaps = {k: 0 for k in [*objects]}
 
 	for key, obj in objects.items():
-		if not isinstance(obj, pd.DataFrame):
-			obj = pd.DataFrame(obj)
+		obj = pd.DataFrame(obj) if not isinstance(obj, pd.DataFrame) else obj
 		available_depth = 0
 		total_value = 0
 
@@ -547,7 +563,7 @@ def calc_single_vwap(objects: dict[str, pd.DataFrame],
 
 		# If the 'min_depth' parameter is set,
 		# then the available depth on the market has to fulfill the minimum requirements.
-		if min_depth and available_depth and available_depth > desired_depth * min_depth\
+		if min_depth and available_depth and available_depth > desired_depth * min_depth \
 				or not min_depth and available_depth:
 			vwaps[key] = round(total_value / available_depth, 2)
 
