@@ -52,13 +52,13 @@ def plot_orderbook(orderbooks: dict[str, dict[str, pd.DataFrame]],
 				   orderbook_key: Union[int, str] = 0,
 				   timestamp: Union[int, str] = -1) -> Union[go.Figure, None]:
 	"""
-	Creates a plotly plot of a single orderbook to be displayed by Dash server. Use orderbook_key to specifiy an
-	orderbook and timestamp to specify the timeframe to display.
+		Creates a plotly plot of a single order book to be displayed by Dash server. Use order book_key to specify an
+		order book and timestamp to specify the timeframe to display.
 
 	Args:
-		orderbooks (dict{key: DataFrame}): Orderbooks
+		orderbooks (dict{key: DataFrame}): Order books
 		orderbook_key (int): Dictionary key
-		timestamp (int): Orderbook Key
+		timestamp (int): Order book Key
 
 	Returns:
 		Plotly plot
