@@ -1,6 +1,6 @@
 import os
 import pickle
-from datetime import datetime
+from datetime import datetime, timezone
 from decimal import Decimal, getcontext
 from pathlib import Path
 from time import sleep
@@ -10,6 +10,7 @@ import pandas as pd
 from powerbot_client import ApiClient, TradesApi, SignalsApi, Signal, Trade, InternalTrade, OrdersApi, OwnOrder
 
 from powerbot_backtesting.utils import *
+from powerbot_backtesting.models import HistoryApiClient
 
 
 def _get_private_data(api_client: ApiClient,
@@ -19,6 +20,22 @@ def _get_private_data(api_client: ApiClient,
 					  delivery_area: str = None,
 					  portfolio_id: list[str] = None,
 					  active_only: bool = False) -> list[Union[InternalTrade, OwnOrder, Trade, Signal]]:
+	"""
+	Underlying function of all private data requests to PowerBot. Loads the specified collection according to the
+	parameters given.
+
+	Args:
+		api_client: PowerBot ApiClient
+		data_type (str): Either internal_trade, own_trade, own_order or signal
+		time_from (str/ datetime): YYYY-MM-DD or YYYY-MM-DD hh:mm:ss
+		time_till (str/ datetime): YYYY-MM-DD or YYYY-MM-DD hh:mm:ss
+		delivery_area (str): EIC Area Code for Delivery Area
+		portfolio_id (list[str]): List of specific portfolio IDs to load trades from. If left out, will load from all IDs.
+		active_only (bool):  True if only active orders should be loaded. If False, loads also hibernate and inactive.
+
+	Returns:
+		list[Union[InternalTrade, OwnOrder, Trade, Signal]]
+	"""
 	param_mapping = {
 		"internal_trade": {"delivery_within_start": time_from, "delivery_within_end": time_till, "limit": 100},
 		"own_trade": {"delivery_within_start": time_from, "delivery_within_end": time_till, "limit": 500},
@@ -58,7 +75,7 @@ def _cache_data(data_type: str,
 				data: dict[str, pd.DataFrame],
 				delivery_area: str,
 				exchange: str = None,
-				api_client: object = None,
+				api_client: Union[ApiClient, HistoryApiClient] = None,
 				timesteps: int = 0,
 				time_unit: str = None,
 				gzip_files: bool = True,
@@ -83,10 +100,10 @@ def _cache_data(data_type: str,
 		as_pickle (bool): False per default, except for orderbooks
 	"""
 	# Setup
-	host = api_client.configuration.host if api_client else None
+	host = api_client.configuration.host if isinstance(api_client, ApiClient) else None
 	environment = "staging" if host and host.split("/")[2].split(".")[0] == "staging" else "prod"
-	exchange = host.split("/")[4] if host else exchange
-	folder = "raw" if data_type in ["trades", "ordhist"] else "processed"
+	exchange = host.split("/")[4] if host else api_client.exchange if isinstance(api_client, HistoryApiClient) else exchange
+	folder = "raw" if data_type in ["trades", "ordhist", "contracts"] else "processed"
 	compression = "gzip" if gzip_files else "infer"
 	file_ending = ".gz" if gzip_files else ""
 
@@ -97,6 +114,7 @@ def _cache_data(data_type: str,
 		day_month = delivery_date.strftime("%m-%d")
 		file_name = key.replace(f"{str(key).split(' ')[0]}", "").replace(":", "-")
 		file_name = f"{file_name}_{data_type}" if folder == "raw" else f"{file_name}_{data_type}_{timesteps}{time_unit}"
+		file_name = f"{delivery_date.date()}{file_name}" if file_name.startswith("_") else file_name
 
 		# Check if __cache__ already exists
 		cache_path = _find_cache().joinpath(
@@ -167,19 +185,21 @@ def _find_cache() -> Path:
 	return cache_path
 
 
-def _get_file_cachepath(api_client: ApiClient, contract_key: str, delivery_area: str) -> str:
+def _get_file_cachepath(api_client: Union[ApiClient, HistoryApiClient], contract_key: str, delivery_area: str) -> str:
 	"""
 	Helper function that constructs most of the path of a cached file.
 
 	Args:
-		api_client: ApiClient
-		contract_key: Key of dictionary
+		api_client: PowerBot ApiClient if loading from API else HistoryApiClient
+		contract_key (str): Key of dictionary
+		delivery_area (str): EIC Area Code for Delivery Area
 
 	Returns:
 		filepath: str
 	"""
-	environment = api_client.configuration.host.split("/")[2].split(".")[0]
-	market = api_client.configuration.host.split("/")[4]
+	environment = api_client.configuration.host.split("/")[2].split(".")[0] if isinstance(api_client, ApiClient) else None
+	environment = "staging" if environment == "staging" else "prod"
+	market = api_client.configuration.host.split("/")[4] if isinstance(api_client, ApiClient) else api_client.exchange
 	delivery_date = datetime.strptime(contract_key.split(" ")[0], DATE_YMD)
 	year_month = delivery_date.strftime(DATE_YM)
 	day_month = delivery_date.strftime(DATE_MD)
@@ -188,7 +208,7 @@ def _get_file_cachepath(api_client: ApiClient, contract_key: str, delivery_area:
 	return f"{environment}\\{market}_{delivery_area}\\{year_month}\\{day_month}\\raw\\{file_name}"
 
 
-def _check_contracts(contract, delivery_areas: list[str], products: list[str]) -> bool:
+def _check_contracts(contract, delivery_areas: list[str], products: list[str], allow_udc: bool) -> bool:
 	"""
 	Helper function to determine if contract is of interest and should be added to contract dictionary.
 
@@ -198,7 +218,7 @@ def _check_contracts(contract, delivery_areas: list[str], products: list[str]) -
 		products: List of products
 
 	Returns:
-		Bool
+		bool
 	"""
 	if delivery_areas and contract.delivery_areas and not any(
 			area in contract.delivery_areas for area in delivery_areas) \
@@ -206,7 +226,8 @@ def _check_contracts(contract, delivery_areas: list[str], products: list[str]) -
 		area in contract.contract_details["deliveryAreas"] for area in delivery_areas) \
 			or delivery_areas and not contract.delivery_areas and not contract.contract_details["deliveryAreas"] \
 			or products and contract.product not in products \
-			or not products and "10YGB----------A" not in delivery_areas and contract.product == "GB_Hour_Power":
+			or not products and "10YGB----------A" not in delivery_areas and contract.product == "GB_Hour_Power"\
+			or contract.type == "UDC" and not allow_udc:
 		return False
 	return True
 
@@ -215,10 +236,10 @@ def _order_matching(order_side: str,
                     orderbook: pd.DataFrame,
                     timestamp: str,
                     price: Union[int, float],
-                    quantity: Union[int, float],
+                    quantity: Union[int, float, Decimal],
                     exec_orders_list: dict[str, int],
                     trade_list: dict[int, dict[str, int]],
-                    contract_time: Union[int, float],
+                    contract_time: int,
                     order_execution: str = "NON") -> Decimal:
 	"""
 	Matches orders according to input parameters; adds trades made to trade_list and returns the remaining quantity.
@@ -231,14 +252,14 @@ def _order_matching(order_side: str,
 	FOK - Fill or Kill, if order isn't filled completely by first matching order, next matching order is loaded ->
 	if none match next order book is loaded
 
-	IOC - Immediate and Cancel, order is executed to maximum extent by first matching order, next orderbook is loaded ->
+	IOC - Immediate and Cancel, order is executed to maximum extent by first matching order, next order book is loaded ->
 	allows price adjustments
 
 	Args:
 		order_side (str): buy/sell
-		orderbook (DataFrame): Single Orderbook
-		timestamp(str): Timestamp of Orderbook
-		price (int): Minimum/ Maxmimum Price for Transaction
+		orderbook (DataFrame): Single order book
+		timestamp (str): Timestamp of order book
+		price (int): Minimum/ Maximum Price for Transaction
 		quantity (int): Quantity to buy/sell
 		exec_orders_list (dict): Dictionary of already matched order IDs
 		trade_list (list): List of executed trades
@@ -246,7 +267,7 @@ def _order_matching(order_side: str,
 		order_execution (str): Type of order execution that should be simulated
 
 	Returns:
-		quantity: remaining quantity as int
+		Decimal: remaining quantity
 	"""
 
 	# Transform Values to Decimals
@@ -319,8 +340,11 @@ def _orderbook_data_transformation(orders: pd.DataFrame) -> pd.DataFrame:
 	"""
 	Function transforms data in passed dataframe to be compatible with process_orderbooks function
 
+	Args:
+		orders (pd.DataFrame): DataFrame containing order data
+
 	Returns:
-		Dataframe
+		pd.Dataframe
 	"""
 	if not isinstance(orders, pd.DataFrame):
 		return pd.DataFrame()
@@ -346,7 +370,7 @@ def _orderbook_data_transformation(orders: pd.DataFrame) -> pd.DataFrame:
 				if row[side] and not isinstance(row[side], float):
 					for entry in row[side]:
 						entry["type"] = side[:-1]
-						entry["as_of"] = row["as_of"]
+						entry["as_of"] = row["as_of"].tz_localize(timezone.utc)
 						entry["delta"] = row["delta"]
 						bids_asks.append(entry)
 
@@ -366,7 +390,7 @@ def _delta_filter(orderbook: pd.DataFrame) -> pd.DataFrame:
 	the last delta: false and going back until the space between two orders that have delta: false is bigger than 1.
 
 	Args:
-		orderbook: pd.Dataframe
+		orderbook (pd.Dataframe): Preliminary order book
 
 	Returns:
 		pd.Dataframe
@@ -387,6 +411,18 @@ def _delta_filter(orderbook: pd.DataFrame) -> pd.DataFrame:
 
 
 def _historic_data_transformation(files: list, exchange: str, filetype: str) -> pd.DataFrame:
+	"""
+	Function transforms historic data into correct format to be used with other data processing functions.
+
+	Args:
+		files (list): List of files to be transformed
+		exchange (str): Exchange e.g. epex, nordpool, southpool, etc.
+		filetype (str): either trades or orders
+
+	Returns:
+		pd.DataFrame
+	"""
+	# Append contracts that are in the same timeframe to one DataFrame
 	df = None
 	for file in files:
 		if not isinstance(df, pd.DataFrame):
